import React from 'react';
import { Meta } from '@storybook/react';

import { SchemaUtils } from '@graphpolaris/shared/lib/schema/schema-utils';
import { schemaSlice, setSchema } from '@graphpolaris/shared/lib/data-access/store';

import { configureStore } from '@reduxjs/toolkit';
import { Provider } from 'react-redux';
import { Schema } from './Schema';

import {
  movieSchemaRaw,
  northwindSchemaRaw,
  simpleSchemaRaw,
  twitterSchemaRaw,
  movieSchema,
  northWindSchema,
  simpleSchema,
  twitterSchema,
} from '@graphpolaris/shared/lib/mock-data';

const Component: Meta<typeof Schema> = {
  /* 👇 The title prop is optional.
   * See https://storybook.js.org/docs/react/configure/overview#configure-story-loading
   * to learn how to generate automatic titles
   */
  title: 'Schema/Panel',

  component: Schema,
  decorators: [
    (story) => (
      <Provider store={Mockstore}>
        <div
          style={{
            width: '100%',
            height: '100vh',
          }}
        >
          {story()}
        </div>
      </Provider>
    ),
  ],
};

// Mock the schema and palette store
const Mockstore = configureStore({
  reducer: {
    schema: schemaSlice.reducer,
  },
});

export const TestSimple = {
  play: async () => {
    const dispatch = Mockstore.dispatch;
    const schema = SchemaUtils.schemaBackend2Graphology({
      nodes: [
        {
          name: 'Thijs',
          attributes: [
            { name: 'city', type: 'string' },
            { name: 'vip', type: 'bool' },
            { name: 'state', type: 'string' },
          ],
        },
        {
          name: 'Airport',
          attributes: [
            { name: 'city', type: 'string' },
            { name: 'vip', type: 'bool' },
            { name: 'state', type: 'string' },
          ],
        },
      ],
      edges: [
        {
          name: 'Thijs:Airport',
          label: 'Thijs:Airport',
          from: 'Thijs',
          to: 'Airport',
          collection: 'flights',
          attributes: [
            { name: 'arrivalTime', type: 'int' },
            { name: 'departureTime', type: 'int' },
          ],
        },
      ],
    });

    dispatch(setSchema(schema.export()));
  },
};

export const TestTooltip = {
  play: async () => {
    const dispatch = Mockstore.dispatch;
    const schema = SchemaUtils.schemaBackend2Graphology({
      nodes: [
        {
          name: 'Thijs',
          attributes: [
            { name: 'city', type: 'string' },
            { name: 'vip', type: 'bool' },
            { name: 'state', type: 'string' },
          ],
        },
      ],
      edges: [],
    });

    dispatch(setSchema(schema.export()));
  },
};

export const TestMovieSchema = {
  play: async () => {
    const dispatch = Mockstore.dispatch;
    const schema = SchemaUtils.schemaBackend2Graphology(movieSchemaRaw);

    dispatch(setSchema(schema.export()));
  },
};

// export const TestWithSchema = {
//   play: async () => {
//     const dispatch = Mockstore.dispatch;
//     const schema = SchemaUtils.ParseSchemaFromBackend({
//       nodes: [
//         {
//           name: 'Thijs',
//           attributes: [],
//         },
//         {
//           name: 'Airport',
//           attributes: [
//             { name: 'city', type: 'string' },
//             { name: 'vip', type: 'bool' },
//             { name: 'state', type: 'string' },
//           ],
//         },
//         {
//           name: 'Airport2',
//           attributes: [
//             { name: 'city', type: 'string' },
//             { name: 'vip', type: 'bool' },
//             { name: 'state', type: 'string' },
//           ],
//         },
//         {
//           name: 'Plane',
//           attributes: [
//             { name: 'type', type: 'string' },
//             { name: 'maxFuelCapacity', type: 'int' },
//           ],
//         },
//         { name: 'Staff', attributes: [] },
//       ],
//       edges: [
//         {
//           name: 'Airport2:Airport',
//           from: 'Airport2',
//           to: 'Airport',
//           collection: 'flights',
//           attributes: [
//             { name: 'arrivalTime', type: 'int' },
//             { name: 'departureTime', type: 'int' },
//           ],
//         },
//         {
//           name: 'Airport:Staff',
//           from: 'Airport',
//           to: 'Staff',
//           collection: 'flights',
//           attributes: [{ name: 'salary', type: 'int' }],
//         },
//         {
//           name: 'Plane:Airport',
//           from: 'Plane',
//           to: 'Airport',
//           collection: 'flights',
//           attributes: [],
//         },
//         {
//           name: 'Airport:Thijs',
//           from: 'Airport',
//           to: 'Thijs',
//           collection: 'flights',
//           attributes: [{ name: 'hallo', type: 'string' }],
//         },
//         {
//           name: 'Thijs:Airport',
//           from: 'Thijs',
//           to: 'Airport',
//           collection: 'flights',
//           attributes: [{ name: 'hallo', type: 'string' }],
//         },
//         {
//           name: 'Staff:Plane',
//           from: 'Staff',
//           to: 'Plane',
//           collection: 'flights',
//           attributes: [{ name: 'hallo', type: 'string' }],
//         },
//         {
//           name: 'Staff:Airport2',
//           from: 'Staff',
//           to: 'Airport2',
//           collection: 'flights',
//           attributes: [{ name: 'hallo', type: 'string' }],
//         },
//         {
//           name: 'Airport2:Plane',
//           from: 'Airport2',
//           to: 'Plane',
//           collection: 'flights',
//           attributes: [{ name: 'hallo', type: 'string' }],
//         },

//         {
//           name: 'Airport:Airport',
//           from: 'Airport',
//           to: 'Airport',
//           collection: 'flights',
//           attributes: [{ name: 'test', type: 'string' }],
//         },
//       ],
//     });

//     console.info('dispatch dummy schema', schema.order);
//     dispatch(setSchema(schema.export()));
//   },
// };

// export const TestWithMockNorthWindSchema = {
//   play: async () => {
//     const dispatch = Mockstore.dispatch;

//     const schema = northWindSchema;
//     console.info('dispatch northWindSchema schema', schema.order);
//     dispatch(setSchema(schema.export()));
//   },
// };

// export const TestWithMockTwitterSchema = {
//   play: async () => {
//     const dispatch = Mockstore.dispatch;

//     const schema = twitterSchema;
//     console.info('dispatch TestWithMockTwitterSchema schema', schema.order);
//     dispatch(setSchema(schema.export()));
//   },
// };

// export const TestWithMockSimpleSchema = {
//   play: async () => {
//     const dispatch = Mockstore.dispatch;

//     const schema = simpleSchema;
//     console.info('dispatch simpleSchema schema', schema.order);
//     dispatch(setSchema(schema.export()));
//   },
// };

// export const TestWithMockMovieSchema = {
//   play: async () => {
//     const dispatch = Mockstore.dispatch;

//     const schema = movieSchema;
//     console.info('dispatch movieSchema schema', schema.order);
//     dispatch(setSchema(schema.export()));
//   },
// };

export default Component;
