import React from 'react';
import { CompositeLayer, HeatmapLayer, Layer } from 'deck.gl';
import * as d3 from 'd3';
import { getDistance } from '../../utlis';
import { CompositeLayerType, Edge, LayerProps } from '../../mapvis.types';
import { Node } from '@graphpolaris/shared/lib/data-access';

export class HeatLayer extends CompositeLayer<CompositeLayerType> {
  static type = 'heatmap';
  private _layers: Record<string, Layer> = {};

  constructor(props: LayerProps) {
    super(props);
  }

  updateState({ changeFlags }: { changeFlags: any }) {
    return changeFlags.propsOrDataChanged || changeFlags.somethingChanged;
  }

  createSegments(edges: Edge[]) {
    // Generates a path between source and target nodes
    return edges.map((edge: Edge, index) => {
      const length = getDistance(edge.path[0], edge.path[1]);
      const nSegments = length * this.props.settings.nSegments;

      let xscale = d3
        .scaleLinear()
        .domain([0, nSegments + 1])
        .range([edge.path[0][0], edge.path[1][0]]);

      let yscale = d3
        .scaleLinear()
        .domain([0, nSegments + 1])
        .range([edge.path[0][1], edge.path[1][1]]);

      let source = edge.path[0];
      let target = null;
      let local = [source];

      for (let j = 1; j <= nSegments; j++) {
        target = [xscale(j), yscale(j)];
        local.push(target);
        source = target;
      }

      local.push(edge.path.slice(-1)[0]);
      return { ...edge, path: local };
    });
  }

  renderLayers() {
    const { data, settings, getNodeLocation, setLayerIds, graphMetadata } = this.props;
    const layerSettings = settings[HeatLayer.type];

    const layerIds: string[] = [];

    graphMetadata.nodes.labels.forEach((label: string) => {
      const layerId = `${label}-nodes-heatmaplayer`;
      layerIds.push(layerId);

      this._layers[layerId] = new HeatmapLayer<Node>({
        id: layerId,
        data: data.nodes.filter((node: Node) => node.label === label),
        visible: !layerSettings.nodes[label].hidden,
        getPosition: (d: any) => getNodeLocation(d.id),
        getWeight: (d: any) => layerSettings.nodes[label].size,
        radiusPixels: layerSettings.nodes[label].size,
        aggregation: 'SUM',
      });
    });

    setLayerIds(layerIds);

    return Object.values(this._layers);
  }
}

HeatLayer.layerName = 'Heatmap';
